#ifndef GEODESIE_H
#define GEODESIE_H

// --- Personalized types -------------------------------------------------------

// Vecteur (translation/rotation)
typedef struct {
   double x;
   double y;
   double z;
} vector;

// Coordonnees cartesiennes
typedef struct {
   double X;
   double Y;
   double Z;
} coord_cartesiennes;

// Coordonnees geographiques
typedef struct {
   double phiv;   // latitude
   double lambda; // longitude
   double he;     // hauteur ellipsoidale
} coord_geographiques;

// Coordonnees planes
typedef struct {
   double N; // latitude
   double E; // longitude
} coord_planes;

// --- Functions prototypes -----------------------------------------------------

// Calcul de la premiere excentricite
double E(double a, double b);

/* Calcul de la latitude isometrique sur un ellipsoide de premiere excentricite e
   au point de latitude phi.
*/
double L(double phiv, double e);

// Calcul la grande normale
double N(double phiv, double a, double e);

/* Transformation, pour un ellipsoide donne, des coordonnees cartesiennes d'un
   point en coordonnees geographiques ellipsoidales par la methode de
   Heiskanen-Moritz-Boucher.
*/
coord_geographiques cc2cg(coord_cartesiennes cc, double a, double e);

/* Transformation de coordonnees geographiques ellipsoidales en coordonnees
   cartesiennes.
*/
coord_cartesiennes cg2cc(coord_geographiques cg, double a, double e);

/* Transformation de coordonnees geographiques en coordonnees planes conforme a
   la projection conique de Lambert.
*/
coord_planes cg2cp(coord_geographiques cg, double C, double e, double lambda0, double n, double Xs, double Ys);

/* Calcul des coordonnees cartesiennes tridimensionnelles dans le systeme (2) a
   partir d'un jeu de 7 parametres (3 translations, 1 facteur d'echelle et 3
   rotations) de passage du systeme (1) vers le systeme (2), et des coordonnees
   cartesiennes tridimensionnelles dans le systeme (1).
*/
coord_cartesiennes similitude3D_7params(coord_cartesiennes cc, vector T, double D, vector R);

#endif
