#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "raf98.h"

double pasj, pasl, raf98[32768][8];

int loadRAF98(const char *gridFile) {
   FILE *pGridFile=fopen(gridFile, "rt");

   if(pGridFile != NULL) {
      char fileLine[1024]=""; // Same as in Notepad

      fgets(fileLine, 1024, pGridFile);
      sscanf(fileLine, "%lf %lf\n", &pasj, &pasl);
      fgets(fileLine, 1024, pGridFile);
      sscanf(fileLine, "%lf %lf\n", &pasj, &pasl);
      fgets(fileLine, 1024, pGridFile);
      sscanf(fileLine, "%lf %lf\n", &pasj, &pasl);

      unsigned short n=4;
      while(fgets(fileLine, 1024, pGridFile)) {
         if(n%53 == 3) {
            sscanf(fileLine, "%lf %lf %lf %lf %lf\n", &raf98[n][0], &raf98[n][1], &raf98[n][2], &raf98[n][3], &raf98[n][4]);
         } else {
            sscanf(fileLine, "%lf %lf %lf %lf %lf %lf %lf %lf\n", &raf98[n][0], &raf98[n][1], &raf98[n][2], &raf98[n][3], &raf98[n][4], &raf98[n][5], &raf98[n][6], &raf98[n][7]);
         }

         n++;
      }

      return true;
   } else {
      return false;
   }
}

double Nraf98(double j, double l) {
   double ipart=0.0;

//   modf(l/pasl, &ipart);
   ipart=floor(l/pasl);
   double lmin=ipart*pasl;
   double lmax=lmin+pasl;
   unsigned short nlmin=(unsigned short)floor(30*lmin+166);
   unsigned short nlmax=(unsigned short)floor(30*lmax+166);

   double jmin=floor(j/pasj)*pasj;
   double jmax=jmin+pasj;
   unsigned short njmin=(unsigned short)floor(-40*jmin+2061);
   unsigned short njmax=(unsigned short)floor(-40*jmax+2061);

   bilinear N[4];

   modf(nlmin-1/8, &ipart);
   modf(ipart/8, &ipart);

   N[0].lgn=(53*njmin-52)+ipart+3;
   N[0].col=((nlmin-1)%8);
   N[0].val=raf98[N[0].lgn][N[0].col];

   N[1].lgn=(53*njmax-52)+ipart+3;
   N[1].col=((nlmin-1)%8);
   N[1].val=raf98[N[1].lgn][N[1].col];

   modf(nlmax-1/8, &ipart);
   modf(ipart/8, &ipart);

   N[2].lgn=(53*njmin-52)+ipart+3;
   N[2].col=((nlmax-1)%8);
   N[2].val=raf98[N[2].lgn][N[2].col];

   N[3].lgn=(53*njmax-52)+ipart+3;
   N[3].col=((nlmax-1)%8);
   N[3].val=raf98[N[3].lgn][N[3].col];

   double X=(l-lmin)/(lmax-lmin);
   double Y=(j-jmin)/(jmax-jmin);

   return (1-X)*(1-Y)*N[0].val+
          (1-X)*Y*N[1].val+
          X*(1-Y)*N[2].val+
          X*Y*N[3].val;
}

double altitude(double lat, double lon, double depth, double he, double ha) {
   return he-(depth+Nraf98(lat, lon)+ha);
}
